;;;                                                                    ;
;;;  atableT2G.LSP                                                     ;
;;;                                                                    ;
;;;  Copyright 2005 by Alexander Shchetinin. All Rights Reserved.      ;
;;;                                                                    ;
;;;  You are hereby granted permission to use, copy and modify this    ;
;;;  software without charge, provided you do so exclusively for       ;
;;;  your own use or for use by others in your organization in the     ;
;;;  performance of their normal duties, and provided further that     ;
;;;  the above copyright notice appears in all copies and both that    ;
;;;  copyright notice and the limited warranty and restricted rights   ;
;;;  notice below appear in all supporting documentation.              ;
;;;                                                                    ;
;;;  Incorporation of any part of this software into other software,   ;
;;;  except when such incorporation is exclusively for your own use    ;
;;;  or for use by others in your organization in the performance of   ;
;;;  their normal duties, is prohibited without the prior written      ;
;;;  consent of author.                                                ;
;;;                                                                    ;
;;;  Copying, modification and distribution of this software or any    ;
;;;  part thereof in any form except as expressly provided herein is   ;
;;;  prohibited without the prior written consent of author.           ;
;;;                                                                    ;
;;;  AUTHOR PROVIDES THIS SOFTWARE "AS IS" AND WITH ALL FAULTS.        ;
;;;  AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF             ;
;;;  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTHOR,         ;
;;;  DOES NOT WARRANT THAT THE OPERATION OF THE SOFTWARE               ;
;;;  WILL BE UNINTERRUPTED OR ERROR FREE.                              ;
;;;                                                                    ;
;;;--------------------------------------------------------------------;
;;;       Function:  atableT2G                                         ;
;;;                                                                    ;
;;;    Description:  This function is copy text from drawing to cell,  ;
;;;                  column or row. You can sort selected text by      ;
;;;                  X or Y.                                           ;
;;;                                                                    ;
;;;                  Required Application: ATable for AutoCAD          ;
;;;                                                                    ;
;;;      Arguments:  None                                              ;
;;;                                                                    ;
;;; Returned Value:  None	                                       ;
;;;                                                                    ;
;;;          Usage:  Load atableT2G.lsp in AutoCAD by command Appload. ;
;;;                  Run ATable for AutoCAD. Display ATable editor     ;
;;;                  Enter option Add-ons in command line. Select T2G  ;
;;;                  option and follow instructions in command line.   ;
;;;                                                                    ;
;;;--------------------------------------------------------------------;

;You can localize this application. Only change text strings below.
(setq atableT2Glocal
  (list 
    (cons 10 "\nSelect text objects.")
    (cons 20 "Alxd ATable is closed or unavailable!")
    (cons 30 "Active sheet of Alxd ATable is closed or unavailable!")
    (cons 60 "XSort")
    (cons 61 "YSort")
    (cons 63 "toCEll")
    (cons 64 "toColumn")
    (cons 65 "toRow")
    (cons 66 "Over")
    (cons 67 "reSelect")
    (cons 69 "eXit")
    (cons 70 "\nEnter an option [XSort/YSort/toCEll/toColumn/toRow/Over/reSelect/eXit]: ")
    (cons 100 "\nCurrent cell: ")
    (cons 101 "\nRows added: ")
    (cons 102 "\nColumns added: ")
    (cons 110 "\nEnitity in selection sorted by X.")
    (cons 111 "\nEnitity in selection sorted by Y.")
    (cons 120 "\nNothing selected.")
    (cons 130 "\nEnter object step")
  )
)

(defun rounddiv ( a1 a2 ) (1+ (/ (1- a1) a2)) )

(defun getLocal ( slocal nlocal / )
  (cdr (assoc nlocal slocal))
)

(defun textGetInsPoint (ename / ent p_10)
  (setq ent (entget ename))
  (setq p_10 (cdr (assoc 10 ent)))
  p_10
);defun

(defun textGetString (ename / ent)
  (setq ent (entget ename))
  (cdr (assoc 1 ent))
);defun

(defun textEntityCompareY (ename1 ename2)
  (>= (cadr (textGetInsPoint ename1)) (cadr (textGetInsPoint ename2))) 
);defun

(defun textEntityCompareX (ename1 ename2)
  (<= (car (textGetInsPoint ename1)) (car (textGetInsPoint ename2))) 
);defun

(defun sset2elist (sset / ename elist)
  (setq sidx 0 elist '())
  (repeat (sslength sset)
    (setq ename (ssname sset sidx))
    (setq elist (cons ename elist))
    (setq sidx (1+ sidx))
  );repeat
  (reverse elist)
);defun  

(defun elist2text ( elist / ename textstring)
  (setq textstring "")
  (foreach ename elist (setq textstring (strcat textstring (textGetString ename) " ")) ) 
)

(defun textSort ( elist xy )
  (if (= xy T)
    (setq elist (vl-sort elist 'textEntityCompareX))
    (setq elist (vl-sort elist 'textEntityCompareY))
  )
  elist
)

(defun text2cell ( elist atActiveSheet / atCells atCell wi wj ssets)
  (setq 
    ssets (vlax-get-property atActiveSheet 'Selections)
    wi (vlax-get-property ssets 'CurrentCellCol)
    wj (vlax-get-property ssets 'CurrentCellRow)
    atCells (vlax-get-property atActiveSheet 'Cells)
    atCell (vlax-get-property atCells 'Items wi wj)
  )
  (princ (strcat (getlocal atableT2Glocal 100) (itoa (1+ wi)) "," (itoa (1+ wj))))
  (vlax-put-property atCell 'Contain (elist2text elist))

  (vlax-release-object atCells)
  (vlax-release-object atCell)
  (vlax-release-object ssets)
)

(defun text2column ( elist step atActiveSheet / atCells atItems atCell wi wj ssets elen elist1 temp temp1 ee)
  (setq 
    ssets (vlax-get-property atActiveSheet 'Selections)
    wi (vlax-get-property ssets 'CurrentCellCol)
    wj (vlax-get-property ssets 'CurrentCellRow)
    atCells (vlax-get-property atActiveSheet 'Cells)
    atItems (vlax-get-property atActiveSheet 'Rows)
    atCell (vlax-get-property atCells 'Items wi wj)
    elen (length elist)
    temp (vlax-get-property atItems 'Count)
    temp (- elen (- temp wj))
    temp (if (< temp 0) 0 temp)
  )  
  (princ (strcat (getlocal atableT2Glocal 100) (itoa (1+ wi)) "," (itoa (1+ wj))))
  (princ (strcat (getlocal atableT2Glocal 101) (itoa temp)))
  (vlax-put-property atItems 'Count (+ (vlax-get-property atItems 'Count) temp))

  (setq temp 0)
  (repeat (rounddiv elen step)
    (setq 
      temp1 0
      elist1 '()
    )
    (repeat step
      (setq ee (nth (+ (* temp step) temp1) elist))
      (if (/= ee nil)
        (setq 
          elist1 (append elist1 (list ee))
          temp1 (1+ temp)
        )
      )
    )
    (setq
      atCell (vlax-get-property atCells 'Items wi (+ wj temp))
      temp (1+ temp)
    )
    (vlax-put-property atCell 'Contain (elist2text elist1))
  )

  (vlax-release-object atCells)
  (vlax-release-object atItems)
  (vlax-release-object atCell)
  (vlax-release-object ssets)
)

(defun text2row ( elist step atActiveSheet / atCells atItems atCell wi wj ssets elen elist1 temp temp1 ee)
  (setq 
    ssets (vlax-get-property atActiveSheet 'Selections)
    wi (vlax-get-property ssets 'CurrentCellCol)
    wj (vlax-get-property ssets 'CurrentCellRow)
    atCells (vlax-get-property atActiveSheet 'Cells)
    atItems (vlax-get-property atActiveSheet 'Columns)
    atCell (vlax-get-property atCells 'Items wi wj)
    elen (length elist)
    temp (vlax-get-property atItems 'Count)
    temp (- elen (- temp wi))
    temp (if (< temp 0) 0 temp)
  )  
  (princ (strcat (getlocal atableT2Glocal 100) (itoa (1+ wi)) "," (itoa (1+ wj))))
  (princ (strcat (getlocal atableT2Glocal 102) (itoa temp)))
  (vlax-put-property atItems 'Count (+ (vlax-get-property atItems 'Count) temp))

  (setq temp 0 temp1 0)
  (repeat (rounddiv elen step)
    (setq 
      temp1 0
      elist1 '()
    )
    (repeat step
      (setq ee (nth (+ (* temp step) temp1) elist))
      (if (/= ee nil)
        (setq 
          elist1 (append elist1 (list ee))
          temp1 (1+ temp)
        )
      ) 
    )
    (setq
      atCell (vlax-get-property atCells 'Items (+ wi temp) wj)
      temp (1+ temp)
    )
    (vlax-put-property atCell 'Contain (elist2text elist1))
  )

  (vlax-release-object atCells)
  (vlax-release-object atItems)
  (vlax-release-object atCell)
  (vlax-release-object ssets)
)

(defun getActiveSheet(atObject / atEditor atSS atActiveSheet wi)
   (setq
      atEditor (vlax-get-property atObject 'AlxdEditor)
      atSS (vlax-get-property atEditor 'AlxdSpreadSheets)
      wi (vlax-get-property atSS 'Active)
      atActiveSheet (vlax-get-property atSS 'Items wi)
   )
  (vlax-release-object atSS)
  (vlax-release-object atEditor)
  atActiveSheet
)

(defun interfaceT2G (atObject sstext / temp xx cmdstr inistr elist atActiveSheet step)

  (setq elist (sset2elist sstext))

  (setq temp T step 1)
  (while temp

    (setq 
      cmdstr (getlocal atableT2Glocal 70)
      inistr (strcat (getlocal atableT2Glocal 60) " " (getlocal atableT2Glocal 61) " " (getlocal atableT2Glocal 63) " " (getlocal atableT2Glocal 64) " " (getlocal atableT2Glocal 65) " " (getlocal atableT2Glocal 66) " " (getlocal atableT2Glocal 67) " " (getlocal atableT2Glocal 69))
    )

    (initget 1 inistr)
    (setq xx (getkword cmdstr))
    (cond
      ((= xx (getlocal atableT2Glocal 60));xsort
        (setq elist (textSort elist T))
        (princ (getlocal atableT2Glocal 110))
      )
      ((= xx (getlocal atableT2Glocal 61));ysort
        (setq elist (textSort elist nil))
        (princ (getlocal atableT2Glocal 111))
      )
      ((= xx (getlocal atableT2Glocal 63));tocell
        (setq atActiveSheet (getActiveSheet atObject))
        (vlax-put-property atActiveSheet 'EditorMode :vlax-false)

        (text2cell elist atActiveSheet)
        (vlax-invoke-method atActiveSheet 'Recalculate)
        (vlax-invoke-method atActiveSheet 'RedrawBlockDefinition 0 0)

        (vlax-release-object atActiveSheet)
      )
      ((= xx (getlocal atableT2Glocal 64));tocolumn
        (setq atActiveSheet (getActiveSheet atObject))
        (vlax-put-property atActiveSheet 'EditorMode :vlax-false)

        (text2column elist step atActiveSheet)
        (vlax-invoke-method atActiveSheet 'Recalculate)
        (vlax-invoke-method atActiveSheet 'RedrawBlockDefinition 0 0)

        (vlax-release-object atActiveSheet)
      )
      ((= xx (getlocal atableT2Glocal 65));torow
        (setq atActiveSheet (getActiveSheet atObject))
        (vlax-put-property atActiveSheet 'EditorMode :vlax-false)

        (text2row elist step atActiveSheet)
        (vlax-invoke-method atActiveSheet 'Recalculate)
        (vlax-invoke-method atActiveSheet 'RedrawBlockDefinition 0 0)

        (vlax-release-object atActiveSheet)
      )
      ((= xx (getlocal atableT2Glocal 66));over
        (initget (+ 2 4))
        (setq cmdstr (strcat (getlocal atableT2Glocal  130) " <" (itoa step) ">: " ))
        (setq xx (getint cmdstr))
        (setq step (if (/= xx nil) xx step))
        (princ step) 
      )
      ((= xx (getlocal atableT2Glocal 67));reselect
        (setq sstext (ssget '((-4 . "<OR") (0 . "TEXT") (0 . "MTEXT") (-4 . "OR>") )))
        (if (= sstext nil)
          (progn
            (princ (getlocal atableT2Glocal 120))
            (setq temp nil)
          )
          (setq elist (sset2elist sstext))
        )
      )
      ((or (= xx nil) (= xx (getlocal atableT2Glocal 69))) (setq temp nil))
    );cond
  )
)

(defun c:atableT2G( / sstext atObject)

  ;get alxd atable object here
  (setq atObject (vlax-get-object "AlxdGrid.AlxdApplication"))

  ;check value
  (if (= atObject nil)
    (progn
      (alert (getlocal atableX2Glocal 20))
      (exit)
    )
  )

;debug string
;  (vlax-dump-object atObject)

  (princ (getlocal atableT2Glocal 10))
  (setq sstext (ssget '((-4 . "<OR") (0 . "TEXT") (0 . "MTEXT") (-4 . "OR>") )))

  (if (= sstext nil)
    (progn
      (princ (getlocal atableT2Glocal 120))
      (exit)
    )
  )

  (vl-load-com)
  (vl-catch-all-apply 'interfaceT2G (list atObject sstext))
  (vlax-release-object atObject) 
 
  (princ)
)
(princ "\natableT2G v1.0 (addon for ATable) - input text from drawing to tables")